@@ -25,8 +25,6 @@ import glob
 import StringIO
 import gzip
 import shutil
-import sys
-import traceback
 
 
 month_names = {"Jan": 1, "Feb": 2, "Mar": 3, "Apr": 4, "May": 5, "Jun": 6,
@@ -353,11 +351,10 @@ def discover_feed(url):
     try:
         icon_url = None
         content_type = None
-        content = page.read()
         if page.info().has_key("Content-Type"):
             content_type = page.info()["Content-Type"]
         if content_type and content_type.find("html") >=0:
-            html = parse_html(content, get_page_encoding(page))
+            html = parse_html(page, get_page_encoding(page))
             base = get_page_base(page, html)
             icon_tag = html.find(name = "link", attrs = {"rel": re.compile("icon", flags = re.I)})
             if icon_tag:
@@ -367,7 +364,7 @@ def discover_feed(url):
                 rss_tag = html.find(name = "link", attrs = {"rel": "alternate", "type": "application/atom+xml"})
             if rss_tag:
                 return get_feed_details(urljoin(base, rss_tag["href"]), icon_url)
-        return get_feed_details(url, icon_url, content)
+        return get_feed_details(url, icon_url, page)
     finally:
         page.close()
 
@@ -441,7 +438,7 @@ class DownloadQueue:
             urllib.urlretrieve(url, filename)
         except Exception, ex:
             if self.__callback:
-                self.__callback(exception = ex)
+                self.__callback("error", ex)
 
 
     def wait_completion(self):
@@ -515,14 +512,9 @@ class Item:
             self.__download_queue = DownloadQueue(self.__report)
 
 
-    def __report(self, action = None, item = None, exception = None):
-        details = None
-        if exception:
-            action = "error"
-            item = exception
-            details = ''.join(traceback.format_tb(sys.exc_info()[2]))
+    def __report(self, action = None, item = None):
         if self.__callback:
-            self.__callback((action, item, None, details))
+            self.__callback((action, item, None))
 
 
     def __calc_the_tag(self, tag):
@@ -588,7 +580,7 @@ class Item:
                     a["href"] = item.save()
                     return a
         except Exception, ex:
-            self.__report(exception = ex)
+            self.__report("error", ex)
 
 
     def __find_single_page(self, html, base, referer):
@@ -601,7 +593,7 @@ class Item:
                     (new_html, new_base, real_url) = self.__get_page_html(url, headers={"Referer": referer})
                     return (new_html, new_base, True)
         except Exception, ex:
-            self.__report(exception = ex)
+            self.__report("error", ex)
         return (html, base, False)
 
 
@@ -615,7 +607,7 @@ class Item:
                     (new_html, new_base, real_url) = self.__get_page_html(url, headers={"Referer": referer})
                     return (new_html, new_base, get_body_or_html(new_html))
         except Exception, ex:
-            self.__report(exception = ex)
+            self.__report("error", ex)
         return (html, base, None)
 
 
@@ -671,7 +663,7 @@ class Item:
                 tag[attr] = self.__download_queue.queue(url, 
                     self.__get_cache_directory(), self.__cache_path)
             except Exception, ex:
-                self.__report(exception = ex)
+                self.__report("error", ex)
 
 
     def __cache_urls(self, tags, attr, base):
@@ -711,8 +703,6 @@ class Item:
         [root.append(tag) for tag in self.__contents]
         if not filename:
             filename = uuid.uuid1().hex + ".html"
-        if not os.path.exists(self.__path):
-            os.makedirs(self.__path)
         f = open(os.path.join(self.__path, filename), "w")
         f.write(html.prettify())
         f.close()
@@ -736,7 +726,7 @@ class Feed:
         print_version_keywords = print_version_keywords_default,
         single_page_keywords = single_page_keywords_default,
         next_page_keywords = next_page_keywords_default,
-        days_to_keep = None, allow_scripts = False, inline_items = False, cache_items = False,
+        delete_before_date = None, allow_scripts = False, inline_items = False, cache_items = False,
         include = None, exclude = None, content_kwargs = None, reformatter_url = None,
         new_window = False, always_clear_cache = False, ignore_images = False):
         """ Constructor.
@@ -751,7 +741,7 @@ class Feed:
         feed_item_template - html used to construct feed item.
         print_version_keywords - a list of phrases to scan html document searching for reference to
         printable version.
-        days_to_keep - days to keep items before delete them.
+        delete_before_date - all items before this date are deleted during feed update.
         allow_scripts - do not remove script tags from item before caching.
         inline_items - inline referenced resources into feed html.
         cache_items - download and cache referenced resources.
@@ -789,8 +779,7 @@ class Feed:
         self.__next_page_keywords = next_page_keywords
         self.__inline_items = inline_items
         self.__cache_items = cache_items
-        if days_to_keep:
-            self.__delete_before_date = date.today() - timedelta(days_to_keep)
+        self.__delete_before_date = delete_before_date
         self.__allow_scripts = allow_scripts
         self.__ignore_images = ignore_images
         self.__include = None
@@ -807,14 +796,9 @@ class Feed:
         self.new_item_count = 0
 
 
-    def __report(self, action = None, item = None, percent = None, exception = None):
-        details = None
-        if exception:
-            action = "error"
-            item = exception
-            details = ''.join(traceback.format_tb(sys.exc_info()[2]))
+    def __report(self, action = None, item = None, percent = None):
         if self.__callback:
-            self.__callback((action, item, percent, details))
+            self.__callback((action, item, percent))
 
 
     def __unquote_special(self, str):
@@ -837,17 +821,19 @@ class Feed:
     def __get_item_link(self, item):
         if item.guid:
             if not item.guid.has_key("ispermalink") or item.guid["ispermalink"].lower() == "true":
-                return self.__unquote_special(item.guid.string)
+                return item.guid.string
         if item.link:
             if item.link.has_key("href"):
-                return self.__unquote_special(item.link["href"]) #Atom uses href to store links
+                return item.link["href"] #Atom uses href to store links
             elif item.link.string:
-                return self.__unquote_special(item.link.string)
+                return item.link.string
             else:
                 # Once there was a feed with url in CData section
                 for node in item.link:
                     if node.string:
-                        return self.__unquote_special(node.string.strip())
+                        str = node.string.strip()
+                        if str:
+                            return str
 
 
     def __get_item_date(self, item):
@@ -1050,16 +1036,16 @@ class Feed:
                     self.__add_item(div, to_process[i], 20 + (i * 60 / count))
                     self.new_item_count += 1
                 except Exception, ex:
-                    self.__report(exception = ex)
+                    self.__report("error", ex)
             self.__report("downloading images", "", 90)
             self.__download_queue.wait_completion()
         except BaseException, ex:
-            self.__report(exception = ex)
+            self.__report("error", ex)
             raise
         finally:
             if self.new_item_count:
                 if not self.__always_clear_cache:
-                    self.__report("deleting", "", 95)
+                    self.__report("deleting", "", 80)
                     self.__delete_before(html)
                 self.__save_html(path, html)
 
