@@ -18,9 +18,10 @@ import webbrowser
 import glob
 import shutil
 import sys
+import dbus
 from BeautifulSoup import BeautifulStoneSoup
 
-version = "0.8.0"
+version = "0.8.1"
 feed_template_default = """
 <html>
     <head>
@@ -664,7 +665,12 @@ class Program:
         if self.__model.browser:
             webbrowser.get("FeedCircuitCustom").open("file://" + url, new_win)
         else:
-            webbrowser.open(url, new_win)
+            #webbrowser.open(url, new_win)
+            browser = bus.get_object("com.nokia.osso_browser", "/com/nokia/osso_browser/request")
+            if new_win:
+                browser.open_new_window(url, dbus_interface = "com.nokia.osso_browser")
+            else:
+                browser.load_url(url, dbus_interface = "com.nokia.osso_browser")
 
 
     def open_feed(self, feed_config):
@@ -926,8 +932,7 @@ class Program:
 
     def update_complete(self):
         if self.__progress_window:
-            self.__progress_window.destroy()
-            self.__progress_window = None
+            self.__progress_window.response(gtk.RESPONSE_NONE)
         self.save_log()
 
 
@@ -1005,8 +1010,12 @@ class Program:
         self.__progress_window = RefreshProgressWindow(self.__window)
         self.__thread = Thread(target = self.update_feeds, args = (feeds_configs,))
         self.__thread.start()
-        if self.__progress_window.run() == gtk.RESPONSE_CANCEL:
-            self.__aborted = True
+        try:
+            if self.__progress_window.run() == gtk.RESPONSE_CANCEL:
+                self.__aborted = True
+        finally:
+            self.__progress_window.destroy()
+            self.__progress_window = None
 
 
     def refresh(self, source):
@@ -1447,6 +1456,9 @@ class FeedListModel(gtk.GenericTreeModel):
 gtk.gdk.threads_init()
 hildon.hildon_init()
 
+global bus
+bus = dbus.SessionBus()
+
 #This is also to be able to start Feed Circuit under the scratchbox
 try:
     import conic
